package storage

import (
	"testing"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/storage"
	"github.com/stretchr/testify/assert"
)

var (
	uuids = []api.UUID{
		api.MustUUIDFromString("5d4c2f36-4732-11e9-b2b2-d359698c414d"),
		api.MustUUIDFromString("69c05dc8-4732-11e9-97ff-13e2d9977fac"),
	}
)

// PasswordStorageFactory is a factory of PasswordStorage
type PasswordStorageFactory (func(bt testing.TB) (storage.PasswordStorage, func()))

// PasswordStorageTests is a test suite that validate a PasswordStorage implementation
func PasswordStorageTests(t *testing.T, factory PasswordStorageFactory) {
	t.Run("set/verify", func(t *testing.T) {
		passwordStorage, clean := factory(t)
		defer clean()

		passwordStorage.Set(uuids[0], "goodpass")

		assert.False(t, passwordStorage.Verify(uuids[0], "badpass"))
		assert.False(t, passwordStorage.Verify(uuids[1], "goodpass"))
		assert.True(t, passwordStorage.Verify(uuids[0], "goodpass"))
	})
	t.Run("update/verify", func(t *testing.T) {
		passwordStorage, clean := factory(t)
		defer clean()

		passwordStorage.Set(uuids[0], "firstpass")
		assert.True(t, passwordStorage.Verify(uuids[0], "firstpass"))

		passwordStorage.Set(uuids[0], "goodpass")

		assert.False(t, passwordStorage.Verify(uuids[0], "badpass"))
		assert.False(t, passwordStorage.Verify(uuids[1], "goodpass"))
		assert.True(t, passwordStorage.Verify(uuids[0], "goodpass"))
	})
	t.Run("reset", func(t *testing.T) {
		passwordStorage, clean := factory(t)
		defer clean()

		passwordStorage.Set(uuids[0], "firstpass")
		passwordStorage.Reset(uuids[0])
		assert.False(t, passwordStorage.Verify(uuids[0], "firstpass"))

		passwordStorage.Set(uuids[0], "goodpass")
		assert.True(t, passwordStorage.Verify(uuids[0], "goodpass"))
		passwordStorage.Reset(uuids[0])
		assert.False(t, passwordStorage.Verify(uuids[0], "goodpass"))
	})
}
