package demo

import (
	"fmt"
	"time"

	xbus "bitbucket.org/orus-io/go-xbus"
)

// NewHelloWorker is a HelloWorker factory
func NewHelloWorker(actor *xbus.Actor) xbus.ActorService {
	return &HelloWorker{}
}

// HelloWorker is a worker that forward incoming envelopes to the default output
// The service parameters are:
// - processing-duration: time to wait between receive and send, in seconds
type HelloWorker struct{}

// Process handles an incoming process
func (w *HelloWorker) Process(apc *xbus.ActorProcessingContext) error {
	er, err := apc.ReadEnvelope("default")
	if err != nil {
		return err
	}
	env, err := er.Complete(time.Second)
	if err != nil {
		return err
	}

	out := apc.OpenOutput("default", er.EventTypes()...)
	for _, event := range env.GetEvents() {
		for _, i := range event.Items {
			if err := out.AddItems(
				event.Type,
				[]byte(fmt.Sprintf("Hello %s", string(i))),
			); err != nil {
				return err
			}
		}
	}
	if err := out.Close(); err != nil {
		return err
	}
	return nil
}

func init() {
	xbus.RegisterActorService("demo.hello", NewHelloWorker)
}
