package demo

import (
	"time"

	"bitbucket.org/orus-io/go-xbus"
)

// NewRelayWorker is a RelayWorker factory
func NewRelayWorker(actor *xbus.Actor) xbus.ActorService {
	return &RelayWorker{
		time.Duration(actor.GetIntSettingD("processing-duration", 1)) * time.Second,
	}
}

// RelayWorker is a worker that forward incoming envelopes to the default output
// The service parameters are:
// - processing-duration: time to wait between receive and send, in seconds
type RelayWorker struct {
	interval time.Duration
}

// Process handles an incoming process
func (w *RelayWorker) Process(apc *xbus.ActorProcessingContext) error {
	er, err := apc.ReadEnvelope("default")
	if err != nil {
		return err
	}
	env, err := er.Complete(time.Second)
	if err != nil {
		return err
	}
	select {
	case <-apc.Context.Done():
	case <-time.After(w.interval):
		if err := apc.ForwardEnvelope("default", env); err != nil {
			apc.Log.Errorf("Something wrong happened. err=%s", err)
		} else {
			apc.Log.Noticef("Successfully forwarded the envelope")
		}
	}
	return nil
}

func init() {
	xbus.RegisterActorService("demo.relay", NewRelayWorker)
}
