package tools

import (
	"strings"
)

const (
	// Alpha contains the letters upper and lower cased
	Alpha = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

	// Num contains the 10 digits
	Num = "0123456789"

	// Dot is the dot char
	Dot = "."

	// Dash is the '-' char
	Dash = "-"

	// Underscore is the '_' char
	Underscore = "_"

	// EventTypeAllowerCharacters constains all the characters allowed in a event
	// type name
	EventTypeAllowerCharacters = Alpha + Num + Dash + Underscore + Dot

	// AccountNameCharacters contains characters allowed in a account name
	AccountNameCharacters = Alpha + Num + Dash + Underscore
)

func validate(allowedCharacters string, value string) bool {
	if len(value) == 0 {
		return false
	}
	for _, c := range value {
		if !strings.Contains(allowedCharacters, string(c)) {
			return false
		}
	}
	return true
}

// ValidateEventType returns true if the passed event type is valid
func ValidateEventType(eventType string) bool {
	return validate(EventTypeAllowerCharacters, eventType)
}

// ValidateAccountName returns true if the passed account name is valid
func ValidateAccountName(name string) bool {
	return validate(AccountNameCharacters, name)
}
